#include "ul.h"

/* ONLY COMPILE THIS FILE FOR GLX/X11 EQUIPPED SYSTEMS */

#ifdef UL_GLX

#include "pw.h"

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <assert.h>

extern "C"
{
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
}

#include <GL/gl.h>
#include <GL/glx.h>

/*
  These are supposed to be defined in /usr/include/GL/glxext.h
  But at time of writing, they were missing.  The OpenGL ARB
  people have been informed by email.
*/

#ifndef GLX_SAMPLE_BUFFERS_ARB
#define GLX_SAMPLE_BUFFERS_ARB 100000
#define GLX_SAMPLES_ARB        100001
#endif


#ifdef HAVE_MOTIF

extern "C"
{
#include <X11/Xm/MwmUtil.h>
}

#else

/* bit definitions for MwmHints.flags */
#define MWM_HINTS_FUNCTIONS	(1L << 0)
#define MWM_HINTS_DECORATIONS	(1L << 1)
#define MWM_HINTS_INPUT_MODE	(1L << 2)
#define MWM_HINTS_STATUS	(1L << 3)

/* bit definitions for MwmHints.decorations */
#define MWM_DECOR_ALL		(1L << 0)
#define MWM_DECOR_BORDER	(1L << 1)
#define MWM_DECOR_RESIZEH	(1L << 2)
#define MWM_DECOR_TITLE		(1L << 3)
#define MWM_DECOR_MENU		(1L << 4)
#define MWM_DECOR_MINIMIZE	(1L << 5)
#define MWM_DECOR_MAXIMIZE	(1L << 6)

typedef struct
{
  unsigned long flags       ;
  unsigned long functions   ;
  unsigned long decorations ;
           long inputMode   ;
  unsigned long status      ;
} PropMotifWmHints ;

#define PROP_MOTIF_WM_HINTS_ELEMENTS	5

#endif

/* For XF86VIDMODE Support */
#ifdef XF86VIDMODE
#include <X11/extensions/xf86vmode.h>   

static struct xf86vidmode_data {
  int vidmode_available;
  XF86VidModeModeInfo orig_mode;
  int orig_viewport_x;
  int orig_viewport_y;
} XF86VidModeData = { 1, { 0 }, 0, 0 };

static void xf86_vidmode_init () ;
#endif

static bool insideCallback = false ;
static bool resizable      = true;
static bool mouse_grabbed  = false;
static bool keyb_grabbed = false;
static int modifiers   = 0 ;
static int origin [2]  = {   0,   0 } ;
static int size   [2]  = { 640, 480 } ;
static int currScreen  = 0 ;
static int currConnect = 0 ;
static int wmCursor    = PW_CURSOR_LEFT ;
static int fsCursor    = PW_CURSOR_LEFT ;

static pwResizeCB     *resizeCB = NULL ;
static pwExitCB       *exitCB   = NULL ;
static pwKeybdFunc    *kbCB     = NULL ;
static pwMouseFunc    *msCB     = NULL ;
static pwMousePosFunc *mpCB     = NULL ;

static Display     *currDisplay = NULL ;
static XVisualInfo *visualInfo  = NULL ;
static Window       currHandle  ;
static Window       wmWindow    ;
static Window       fsWindow    = None ;
static GLXContext   currContext ;
static Window       rootWindow  ;
static Atom         delWinAtom  ;
static XTextProperty *titlePropertyPtr = NULL;

static bool autoRepeat = false ;
static bool haveFocus = false ;

static void getEvents ();

void pwSetAutoRepeatKey ( bool enable )
{
  if ( (autoRepeat != enable) && haveFocus )
  {
    if ( enable )
      XAutoRepeatOn ( currDisplay ) ;
    else
      XAutoRepeatOff ( currDisplay ) ;
  }
  autoRepeat = enable ;
}


struct PixelFormat
{
  int num_samples ;
  int bits_per_pixel ; ;
  int z_bits ;
} ;


static PixelFormat preferred_pixel_formats [] =
{
  /* NumSamples, RGB_bits, Z_bits */

  {  0, 24, 24 },  /* Progressively nastier image formats */
  {  0, 16, 24 },
  {  0, 16, 16 },
  {  0,  3, 16 },
  {  0,  3,  1 },
  { -1, -1, -1 }   /* Magic end marker */
} ;


void defaultExitFunc ()
{
  pwCleanup () ;
  exit ( 0 ) ;
}


static void chooseVisual ( PixelFormat *pf )
{
  int attribs [ 100 ] ;
  int n = 0 ;

  attribs [n++] = GLX_RGBA ;

  switch ( pf->bits_per_pixel )
  {
    case 3 :
      attribs [n++] = GLX_RED_SIZE   ; attribs [n++] = 1 ;
      attribs [n++] = GLX_GREEN_SIZE ; attribs [n++] = 1 ;
      attribs [n++] = GLX_BLUE_SIZE  ; attribs [n++] = 1 ;
      break ;

    case 16 :
      attribs [n++] = GLX_RED_SIZE   ; attribs [n++] = 5 ;
      attribs [n++] = GLX_GREEN_SIZE ; attribs [n++] = 6 ;
      attribs [n++] = GLX_BLUE_SIZE  ; attribs [n++] = 5 ;
      break ;

    case 24 :
      attribs [n++] = GLX_RED_SIZE   ; attribs [n++] = 8 ;
      attribs [n++] = GLX_GREEN_SIZE ; attribs [n++] = 8 ;
      attribs [n++] = GLX_BLUE_SIZE  ; attribs [n++] = 8 ;
      break ;
  }

  switch ( pf->z_bits )
  {
    case  1 : attribs [n++] = GLX_DEPTH_SIZE ; attribs [n++] =  1 ; break ; 
    case 16 : attribs [n++] = GLX_DEPTH_SIZE ; attribs [n++] = 16 ; break ; 
    case 24 : attribs [n++] = GLX_DEPTH_SIZE ; attribs [n++] = 24 ; break ;
    case 32 : attribs [n++] = GLX_DEPTH_SIZE ; attribs [n++] = 32 ; break ;
  }

  if ( pf->num_samples > 0 )
  {
    attribs [n++] = GLX_SAMPLE_BUFFERS_ARB ; attribs [n++] = 1 ;
    attribs [n++] = GLX_SAMPLES_ARB        ; attribs [n++] = pf->num_samples ;
  }

  attribs [n++] = GLX_DOUBLEBUFFER ;
  attribs [n++] = None ;

  visualInfo = glXChooseVisual ( currDisplay, currScreen, attribs ) ;
}


static void pwRealSetResizable ( bool enable, int width, int height )
{
  XSizeHints         sizeHints ;
  XWMHints             wmHints ;

  resizable = enable;

  if (resizable)
    sizeHints.flags     = USSize;
  else
    sizeHints.flags     = PSize | PMinSize | PMaxSize;;

  if ( origin[0] >= 0 && origin[1] >= 0 )
    sizeHints.flags |= USPosition ;

  sizeHints.x           = origin[0] ;
  sizeHints.y           = origin[1] ;
  sizeHints.width       = width ;
  sizeHints.height      = height ;
  sizeHints.min_width   = width ;
  sizeHints.min_height  = height ;
  sizeHints.base_width  = width ;
  sizeHints.base_height = height ;
  sizeHints.max_width   = width ;
  sizeHints.max_height  = height ;

  wmHints.flags         = StateHint;
  wmHints.initial_state = NormalState ;

  XSetWMProperties ( currDisplay, wmWindow,
	                  titlePropertyPtr, titlePropertyPtr, 0, 0,
                          &sizeHints, &wmHints, NULL ) ;
}


void pwInit ( int multisample, int num_samples )
{
  pwInit ( 0, 0, -1, -1, multisample, "NoName", FALSE, num_samples ) ;
}


void pwSetCallbacks ( pwKeybdFunc *kb, pwMouseFunc *ms,
                      pwMousePosFunc *mp, pwResizeCB *rcb,
                      pwExitCB *ecb )
{
  if ( ! currDisplay )
  {
    fprintf ( stderr, "PW: You must not call pwSetCallbacks before pwInit.\n");
    exit ( 1 ) ;
  }

  kbCB = kb ;
  msCB = ms ;
  mpCB = mp ;
  resizeCB = rcb ;
  exitCB   = ecb ? ecb : defaultExitFunc ;
}


void pwInit ( int x, int y, int w, int h, int multisample,
              char *title, int border, int num_samples )
{
  char *displayName = getenv ( "DISPLAY" ) ;

  if ( displayName == NULL ) displayName = ":0.0" ;

  currDisplay = XOpenDisplay ( displayName ) ;

  if ( currDisplay == NULL )
  {
    fprintf ( stderr, "PW: Can't open display '%s'",
                            XDisplayName ( displayName ) ) ;
    exit ( 1 ) ;
  }


  /* OpenGL GLX extension availability? */

  if ( ! glXQueryExtension ( currDisplay, NULL, NULL ) )
  {
    fprintf ( stderr, "PW: GLX extension not available on display '%s'?!?",
                                             XDisplayName ( displayName ) ) ;
    XCloseDisplay ( currDisplay ) ;
    exit ( 1 ) ;
  }

  currScreen   = DefaultScreen    ( currDisplay ) ;
  rootWindow   = RootWindow       ( currDisplay, currScreen ) ;
  currConnect  = ConnectionNumber ( currDisplay ) ;
  delWinAtom   = XInternAtom      ( currDisplay, "WM_DELETE_WINDOW", 0 ) ;
  fsWindow     = None;
  
#ifdef XF86VIDMODE
  xf86_vidmode_init () ;
#endif

  if ( w == -1 ) w = DisplayWidth  ( currDisplay, currScreen ) ;
  if ( h == -1 ) h = DisplayHeight ( currDisplay, currScreen ) ;

  origin [ 0 ] = x ;
  origin [ 1 ] = y ;
  size   [ 0 ] = w ;
  size   [ 1 ] = h ;

  static XTextProperty titleProperty ;
  XSetWindowAttributes attribs ;
  unsigned int            mask ;
  PixelFormat               pf ;

  for ( int i = 0 ; preferred_pixel_formats [ i ] . num_samples >= 0 ; i++ )
  {
    pf = preferred_pixel_formats [ i ] ;
    pf . num_samples = num_samples ;

    chooseVisual ( &pf ) ;

    if ( visualInfo != NULL )
      break ;
  }

  if ( visualInfo == NULL )
  {
    num_samples = 0 ;

    for ( int i = 0 ; preferred_pixel_formats [ i ] . num_samples >= 0 ; i++ )
    {
      pf = preferred_pixel_formats [ i ] ;
      pf . num_samples = num_samples ;

      chooseVisual ( &pf ) ;

      if ( visualInfo != NULL )
        break ;
    }

    if ( visualInfo == NULL )
    {
      fprintf ( stderr, "PW: Unable to open a suitable graphics window,\n" ) ;
      XCloseDisplay ( currDisplay ) ;
      exit ( 1 ) ;
    }
  }

  attribs.event_mask = StructureNotifyMask | ExposureMask         |
                       ButtonPressMask     | ButtonReleaseMask    |
                       KeyPressMask        | KeyReleaseMask       |
		       EnterWindowMask     | LeaveWindowMask      |
                       PointerMotionMask   | ButtonMotionMask     |
                       VisibilityChangeMask| FocusChangeMask;

  attribs.background_pixmap = None ;
  attribs.background_pixel  = 0    ;
  attribs.border_pixel      = 0    ;

  attribs.colormap = XCreateColormap ( currDisplay, rootWindow,
                                       visualInfo->visual, AllocNone ) ;

  mask = CWBackPixmap | CWBorderPixel | CWColormap | CWEventMask;

  wmWindow = XCreateWindow ( currDisplay, rootWindow,
                           x, y, w, h, 0, visualInfo->depth,
                           InputOutput, visualInfo->visual,
                           mask, &attribs ) ;

  currHandle = XCreateWindow ( currDisplay, wmWindow,
                           x, y, w, h, 0, visualInfo->depth,
                           InputOutput, visualInfo->visual,
                           mask, &attribs ) ;

  currContext = glXCreateContext ( currDisplay, visualInfo, NULL, 1 ) ;

  glXMakeCurrent ( currDisplay, currHandle, currContext ) ;

  if ( ! glXIsDirect ( currDisplay, glXGetCurrentContext() ) )
  {
    fprintf ( stderr,"PW: This is an *INDIRECT* rendering context.") ;
    fprintf ( stderr,"PW: That may be bad for performance." ) ;
  }

  PropMotifWmHints hints  ;
  Atom prop_t ;
  Atom prop   ;

  hints . flags = MWM_HINTS_DECORATIONS ;
  hints . decorations = border ? MWM_DECOR_ALL : 0 ;

  prop_t = prop = XInternAtom ( currDisplay, "_MOTIF_WM_HINTS", True ) ;

  if ( prop != 0 )
    XChangeProperty ( currDisplay, wmWindow, prop, prop_t, 32,
                      PropModeReplace, (unsigned char *) &hints,
                      PROP_MOTIF_WM_HINTS_ELEMENTS) ;

  if (XStringListToTextProperty ( (char **) &title, 1, &titleProperty ) )
    titlePropertyPtr = &titleProperty;

  pwRealSetResizable   ( true, w, h );
  XSetWMProtocols  ( currDisplay, wmWindow, &delWinAtom  , 1 );
  XMapWindow       ( currDisplay, currHandle ) ;
  XMapWindow       ( currDisplay, wmWindow ) ;
  glXMakeCurrent   ( currDisplay, currHandle, currContext ) ;

  haveFocus = true;
  if (!autoRepeat)
    XAutoRepeatOff ( currDisplay ) ;

  pwSetCursor ( PW_CURSOR_LEFT ) ;

#ifdef GL_MULTISAMPLE_FILTER_HINT_NV
  glHint ( GL_MULTISAMPLE_FILTER_HINT_NV, multisample ) ;
#endif

  atexit( pwCleanup ) ;

  kbCB     = NULL ;
  msCB     = NULL ;
  mpCB     = NULL ;
  resizeCB = NULL ;
  exitCB   = defaultExitFunc ;

  insideCallback = false ;

  glClear ( GL_COLOR_BUFFER_BIT ) ;
  pwSwapBuffers () ;
  glClear ( GL_COLOR_BUFFER_BIT ) ;
  pwSwapBuffers () ;
}


void pwSetResizable ( bool enable )
{
  /* Make sure we get any last resize events before doing this */
  getEvents();
  pwRealSetResizable ( enable, size[0], size[1] ) ;
}


void pwGetSize ( int *w, int *h )
{
  if ( w ) *w = size[0] ;
  if ( h ) *h = size[1] ;
}


void pwRealSetCursor ( int c )
{
  int newCursor;

  switch ( c )
  {
    case PW_CURSOR_RIGHT  : newCursor = XC_right_ptr      ; break ;
    case PW_CURSOR_LEFT   : newCursor = XC_left_ptr       ; break ;
    case PW_CURSOR_QUERY  : newCursor = XC_question_arrow ; break ;
    case PW_CURSOR_AIM    : newCursor = XC_target         ; break ;
    case PW_CURSOR_CIRCLE : newCursor = XC_circle         ; break ;
    case PW_CURSOR_WAIT   : newCursor = XC_watch          ; break ;
    case PW_CURSOR_CROSS  : newCursor = XC_crosshair      ; break ;

    case PW_CURSOR_NONE  :
    default:
      {
        Pixmap pix ;
        char   blank_cursor [ 16*16 ] = { 0 } ;
        XColor bcol = { 0 } ;
  
        pix = XCreateBitmapFromData ( currDisplay,
                                      rootWindow,
                                      blank_cursor, 16, 16 ) ;
        XDefineCursor ( currDisplay, currHandle,
                        XCreatePixmapCursor ( currDisplay,
                            pix, pix, &bcol, &bcol, 0, 0 ) ) ;
        XFreePixmap   ( currDisplay, pix ) ;
      }
      return ;
  }
 
  XDefineCursor( currDisplay, currHandle,
                 XCreateFontCursor ( currDisplay, newCursor ) ) ;
}


void pwSetCursor ( int c )
{
  pwRealSetCursor (c);
  wmCursor = c;
  fsCursor = c;
}


void pwSetCursorEx ( int wm, int fs )
{
  if (fsWindow == None)
    pwRealSetCursor (wm);
  else
    pwRealSetCursor (fs);
  wmCursor = wm;
  fsCursor = fs;
}


#ifdef XF86VIDMODE      
static void
xf86_vidmode_init ( void )
{
  int i,j;
  XF86VidModeModeLine *l = (XF86VidModeModeLine *)((char *)
    &XF86VidModeData.orig_mode + sizeof XF86VidModeData.orig_mode.dotclock);
  
  if (!XF86VidModeQueryVersion(currDisplay, &i, &j))
    XF86VidModeData.vidmode_available = 0;
  else if (!XF86VidModeQueryExtension(currDisplay, &i, &j))
    XF86VidModeData.vidmode_available = 0;
  else if (!XF86VidModeGetModeLine(currDisplay, currScreen,
            (int *)&XF86VidModeData.orig_mode.dotclock, l))
    XF86VidModeData.vidmode_available = 0;
  else if (!XF86VidModeGetViewPort(currDisplay, currScreen,
            &XF86VidModeData.orig_viewport_x,
            &XF86VidModeData.orig_viewport_y))
    XF86VidModeData.vidmode_available = 0;
  
  if (!XF86VidModeData.vidmode_available)
    fprintf(stderr, "Warning: XF86VidMode not available\n");
}


/* qsort comparison function for sorting the modes */
static int cmpmodes(const void *va, const void *vb)
{
    const XF86VidModeModeInfo *a = *(const XF86VidModeModeInfo **)va;
    const XF86VidModeModeInfo *b = *(const XF86VidModeModeInfo **)vb;
    if ( a->hdisplay == b->hdisplay )
        return b->vdisplay - a->vdisplay;  
    else
        return b->hdisplay - a->hdisplay;
}


static void pwXf86VmSetFullScreenResolution (int desired_width,
  int desired_height, int &result_width, int &result_height)
{
  int i;
  XF86VidModeModeLine mode ; /* = { .hdisplay = 0, .privsize = 0 } ; */
  XF86VidModeModeInfo **modes;
  int mode_count;
  
  /* C++ can't do struct initialization by struct member name? */
  mode.hdisplay = 0;
  mode.privsize = 0;

  if ( ! XF86VidModeData.vidmode_available )
    return;
  
  /* Try to switch video mode. This must be done after the pointer is
     grabbed, because otherwise it can be outside the window negating the
     XF86VidModeSetViewPort. */
  if (XF86VidModeGetModeLine(currDisplay, currScreen, &i, &mode))
  {
    result_width  = mode.hdisplay;
    result_height = mode.vdisplay;
  }
    
  /* Get list of modelines.  */
  if (!XF86VidModeGetAllModeLines(currDisplay, currScreen, &mode_count, &modes))
          mode_count = 0;
     
  /* Search for an exact matching video mode.  */
  for (i = 0; i < mode_count; i++) {
     if ((modes[i]->hdisplay == desired_width) && 
         (modes[i]->vdisplay == desired_height))
       break;
  }

  /* Search for a non exact match (smallest bigger res). */
  if (i == mode_count) {
    int best_width = 0, best_height = 0;
    qsort(modes, mode_count, sizeof(void *), cmpmodes);
    for (i = mode_count-1; i > 0; i--) {
       if ( ! best_width ) {
          if ( (modes[i]->hdisplay >= desired_width) &&
               (modes[i]->vdisplay >= desired_height) ) {
             best_width = modes[i]->hdisplay;
             best_height = modes[i]->vdisplay;
          }
       } else {
          if ( (modes[i]->hdisplay != best_width) ||
               (modes[i]->vdisplay != best_height) ) {
             i++;
             break;
          }
       }
    }
  }
      
  /* Switch video mode.  */
  if ((i >= 0) &&
      ((modes[i]->hdisplay != mode.hdisplay) ||
       (modes[i]->vdisplay != mode.vdisplay)) &&
      XF86VidModeSwitchToMode(currDisplay, currScreen, modes[i]))
  {
    result_width  = modes[i]->hdisplay;
    result_height = modes[i]->vdisplay;
  }
  
  /* Lock mode switching.  */
  XF86VidModeLockModeSwitch(currDisplay, currScreen, True);
         
  /* Set viewport. */
  XF86VidModeSetViewPort(currDisplay, currScreen, 0, 0);

  /* clean up */
  if (mode.privsize)
    XFree(mode.c_private);

  if (mode_count)
  {
    for (i = 0; i < mode_count; i++)
      if (modes[i]->privsize)
        XFree(modes[i]->c_private);
    XFree(modes);
  }
}


static void pwXf86VmRestoreOriginalResolution ( )
{
  int i;
  XF86VidModeModeLine mode;

  if ( ! XF86VidModeData.vidmode_available )
    return;
  
  /* Unlock mode switching.  */
  XF86VidModeLockModeSwitch(currDisplay, currScreen, False);

  if (!XF86VidModeGetModeLine(currDisplay, currScreen, &i, &mode) ||
      (mode.hdisplay != XF86VidModeData.orig_mode.hdisplay) ||
      (mode.vdisplay != XF86VidModeData.orig_mode.vdisplay))
  {
    if (!XF86VidModeSwitchToMode(currDisplay, currScreen,
          &XF86VidModeData.orig_mode))
      fprintf(stderr, "XF86VidMode couldnot restore original resolution\n");
  }
  if (XF86VidModeData.orig_viewport_x || XF86VidModeData.orig_viewport_y)
  {
    if (!XF86VidModeSetViewPort(currDisplay, currScreen,
          XF86VidModeData.orig_viewport_x,
          XF86VidModeData.orig_viewport_y))
      fprintf(stderr, "XF86VidMode couldnot restore original viewport\n");
  }
}
#endif


void 
pwSetSize ( int w, int h )
{
  bool notResizable = !resizable;
  
  if (notResizable)
    pwRealSetResizable ( true, w, h );

  XResizeWindow ( currDisplay, currHandle, w, h ) ;
  XResizeWindow ( currDisplay, wmWindow, w, h ) ;

  if (notResizable)
    pwRealSetResizable ( false, w, h );

#ifdef XF86VIDMODE      
  if (fsWindow != None)
  {
    int screen_w = 0, screen_h ;
    pwXf86VmSetFullScreenResolution ( w, h, screen_w, screen_h ) ;
    /* check pwXf86VmSetFullScreenResolution didn't fail completly. */
    if (screen_w)
    {
      /* Center the window */
      XMoveWindow ( currDisplay, currHandle, ( screen_w - w ) / 2,
                   ( screen_h - h ) / 2 ) ;
    }
  }
#endif

  XFlush ( currDisplay );
}


void pwSetOrigin ( int x, int y )
{
  XMoveWindow ( currDisplay, wmWindow, x, y ) ;
  XFlush      ( currDisplay ) ;
}


void pwSetSizeOrigin ( int x, int y, int w, int h )
{
  XMoveWindow   ( currDisplay, wmWindow, x, y ) ;
  pwSetSize     ( w, h );
}


static void refreshModifiers( XEvent *event )
{
  modifiers = 0 ;

  if( event->xkey.state & ( ShiftMask | LockMask ) ) modifiers |= PW_SHIFT ;
  if( event->xkey.state & ControlMask              ) modifiers |= PW_CTRL  ;
  if( event->xkey.state & Mod1Mask                 ) modifiers |= PW_ALT   ;
}


int pwGetModifiers ()
{
  return modifiers ;
}


static void getEvents ()
{
  XEvent event ;

  insideCallback = true ;

  while ( XPending ( currDisplay ) )
  {
    int updown = PW_DOWN ;

    XNextEvent ( currDisplay, &event ) ;

    refreshModifiers ( &event ) ;

    switch ( event.type )
    {
      case ClientMessage   : (*exitCB)() ; break ;

      case DestroyNotify   : 
        if ( event.xdestroywindow.window == wmWindow )
          (*exitCB)() ;
        break ;

      case ConfigureNotify :
        if ( currHandle == event.xconfigure.window &&
              ( size[0] != event.xconfigure.width ||
                size[1] != event.xconfigure.height ) )
        {
          size[0] = event.xconfigure.width ;
          size[1] = event.xconfigure.height ;

          glXMakeCurrent ( currDisplay, currHandle, currContext ) ;
          glXWaitX    () ;

          if ( resizeCB != NULL )
            (*resizeCB) ( size[0], size[1] ) ;
        }
	break;

      case MappingNotify:
	XRefreshKeyboardMapping ( (XMappingEvent *) &event ) ;
	break;

      case EnterNotify     :
      case LeaveNotify     :
      case VisibilityNotify:
      case Expose          : break ;

      case MotionNotify    :
        if ( mpCB )
          (*mpCB) ( event.xmotion.x, event.xmotion.y ) ;
        break ;

      case FocusIn:
        if ( !haveFocus && !autoRepeat )
          XAutoRepeatOff ( currDisplay ) ;
        haveFocus = true;
        break;

      case FocusOut:
        if ( haveFocus && !autoRepeat )
          XAutoRepeatOn ( currDisplay ) ;
        haveFocus = false;
        break;

      case ButtonRelease   :
        updown = PW_UP ;
        /* FALLTHROUGH */

      case ButtonPress     :
        {
          int button = -1 ;

          switch ( event.xbutton.button )
          {
            case 1 : button = PW_LEFT_BUTTON   ; break ;
            case 2 : button = PW_MIDDLE_BUTTON ; break ;
            case 3 : button = PW_RIGHT_BUTTON  ; break ;
          }

          if ( button != -1 && msCB )
            (*msCB) ( button, updown, event.xbutton.x, event.xbutton.y ) ;
        }
        break ;

      case KeyRelease      :
        updown = PW_UP ;
        /* FALLTHROUGH */

      case KeyPress        :
        XComposeStatus composeStatus ;
        char           asciiCode [ 32 ] ;
        KeySym         keySym ;

        int len = XLookupString( &event.xkey, asciiCode, sizeof(asciiCode),
                                 &keySym, &composeStatus ) ;
        int result = -1 ;

        if( len > 0 )
          result = asciiCode[ 0 ] ;
        else
        {
          switch( keySym )
          {
            case XK_F1:     result = PW_KEY_F1;     break;
            case XK_F2:     result = PW_KEY_F2;     break;
            case XK_F3:     result = PW_KEY_F3;     break;
            case XK_F4:     result = PW_KEY_F4;     break;
            case XK_F5:     result = PW_KEY_F5;     break;
            case XK_F6:     result = PW_KEY_F6;     break;
            case XK_F7:     result = PW_KEY_F7;     break;
            case XK_F8:     result = PW_KEY_F8;     break;
            case XK_F9:     result = PW_KEY_F9;     break;
            case XK_F10:    result = PW_KEY_F10;    break;
            case XK_F11:    result = PW_KEY_F11;    break;
            case XK_F12:    result = PW_KEY_F12;    break;

            case XK_Left:   result = PW_KEY_LEFT;   break;
            case XK_Right:  result = PW_KEY_RIGHT;  break;
            case XK_Up:     result = PW_KEY_UP;     break;
            case XK_Down:   result = PW_KEY_DOWN;   break;

            case XK_KP_Prior:
            case XK_Prior:  result = PW_KEY_PAGE_UP; break;
            case XK_KP_Next:
            case XK_Next:   result = PW_KEY_PAGE_DOWN; break;
            case XK_KP_Home:
            case XK_Home:   result = PW_KEY_HOME;   break;
            case XK_KP_End:
            case XK_End:    result = PW_KEY_END;    break;
            case XK_KP_Insert:
            case XK_Insert: result = PW_KEY_INSERT; break;
          }
        }

        if ( result != -1 && kbCB != NULL )
          (*kbCB) ( result, updown, event.xkey.x, event.xkey.y ) ;

        break ;
    }
  }

  insideCallback = false ;

  glXMakeCurrent ( currDisplay, currHandle, currContext ) ;
}


void pwSwapBuffers ()
{
  if ( ! currDisplay )
  {
    fprintf ( stderr, "PLIB/PW: FATAL - Application called pwSwapBuffers"
                      " before pwInit.\n" ) ;
    exit ( 1 ) ;
  }

  if ( insideCallback )
  {
    fprintf ( stderr, "PLIB/PW: FATAL - Application called pwSwapBuffers"
                      " from inside a callback function.\n" ) ;
    exit ( 1 ) ;
  }

  glFlush () ;
  glXSwapBuffers ( currDisplay, currHandle ) ;
  getEvents () ;
}


#ifdef NEED_GAMMA
void pwSetGamma ( float g )
{
  XF86VidModeGamma gamma ;
  gamma.red = gamma.green = gamma.blue = g ;
  XF86VidModeSetGamma ( currDisplay, currScreen, & gamma ) ;
}
#endif


void pwSetFullscreen ()
{
  XEvent event;
  XSetWindowAttributes setattr;
  /* local width and height vars used for fullscreen window size and for
     storing the video_mode size which is then used to center the window */
  int fs_width  = DisplayWidth(currDisplay, currScreen);
  int fs_height = DisplayHeight(currDisplay, currScreen);

  /* Grab the keyboard */
  if (XGrabKeyboard(currDisplay, rootWindow, False, GrabModeAsync,
        GrabModeAsync, CurrentTime) != GrabSuccess)
  {
    fprintf(stderr, "Error can not grab keyboard, not going fullscreen\n");
    return;
  }
  keyb_grabbed = true;
  
  /* Create the fullscreen window */
  setattr.override_redirect = True;
  setattr.background_pixel = XBlackPixel(currDisplay, currScreen);
  setattr.border_pixel = XBlackPixel(currDisplay, currScreen);
  setattr.event_mask = StructureNotifyMask;
  setattr.colormap = XCreateColormap ( currDisplay, rootWindow,
                                       visualInfo->visual, AllocNone ) ;
  fsWindow = XCreateWindow(currDisplay, rootWindow,
                               0, 0, fs_width, fs_height, 0,
                               visualInfo->depth, InputOutput,
                               visualInfo->visual, CWOverrideRedirect |
                               CWBackPixel | CWColormap | CWBorderPixel |
                               CWEventMask, &setattr);

  /* Map the fullscreen window */
  XMapRaised(currDisplay, fsWindow);
  /* wait until we are mapped. (shamelessly borrowed from SDL) */
  do {
    XMaskEvent(currDisplay, StructureNotifyMask, &event); 
  } while ( (event.type != MapNotify) || 
            (event.xmap.event != fsWindow) );
  /* Make sure we got to the top of the window stack */
  XRaiseWindow(currDisplay, fsWindow);
  
  /* Reparent the real window */
  XReparentWindow(currDisplay, currHandle, fsWindow, 0, 0);

  /* Grab the mouse.  */
  if (XGrabPointer(currDisplay, currHandle, False,
        PointerMotionMask | ButtonPressMask | ButtonReleaseMask,
        GrabModeAsync, GrabModeAsync, currHandle, None,
        CurrentTime) == GrabSuccess)
    mouse_grabbed = true;
  else
    fprintf(stderr, "Warning can not grab mouse\n");

  /* Make sure we get any last resize events before using size[] */
  getEvents();

#ifdef XF86VIDMODE
  /* Switch resolution */
  pwXf86VmSetFullScreenResolution (size[0], size[1], fs_width, fs_height);
#endif
  
  /* Center the window (if nescesarry) */
  if ((fs_width != size[0]) || (fs_height != size[1]))
    XMoveWindow(currDisplay, currHandle, (fs_width - size[0]) / 2,
                 (fs_height - size[1]) / 2);
                 
  /* And last set the cursor */
  pwRealSetCursor (fsCursor);
}


void pwSetWindowed ()
{
  if (mouse_grabbed) {
    XUngrabPointer(currDisplay, CurrentTime);
    mouse_grabbed = false;
  }
  if (keyb_grabbed) {
    XUngrabKeyboard(currDisplay, CurrentTime);
    keyb_grabbed = false;
  }

#ifdef XF86VIDMODE
  /* Switch resolution */
  pwXf86VmRestoreOriginalResolution ( ) ;
#endif

  /* Reparent the real window! */
  XReparentWindow(currDisplay, currHandle, wmWindow, 0, 0);
  XUnmapWindow(currDisplay, fsWindow);
  XDestroyWindow(currDisplay, fsWindow);
  fsWindow = None;

  /* And last set the cursor */
  pwRealSetCursor (wmCursor);
}


void pwToggleFullscreen ()
{
  if (fsWindow == None)
    pwSetFullscreen () ;
  else
    pwSetWindowed () ;
}


void pwCleanup ()
{
  if ( ! currDisplay )
    return;

  if ( titlePropertyPtr )
  {
    XFree ( titlePropertyPtr->value ) ;
    titlePropertyPtr = NULL;
  }

  if ( fsWindow != None )
    pwSetWindowed ( ) ;

  if ( haveFocus && !autoRepeat )
    XAutoRepeatOn ( currDisplay ) ;

  glXDestroyContext ( currDisplay, currContext ) ;
  XDestroyWindow    ( currDisplay, currHandle  ) ;
  XDestroyWindow    ( currDisplay, wmWindow    ) ;
  XCloseDisplay     ( currDisplay ) ;
  haveFocus = false;
  currDisplay = NULL;
}


#endif

